import React, { useCallback } from "react";
import SwaggerClient from 'swagger-client';
import { createContext, useContext, useEffect, useState } from "react";
import { CircularProgress } from '@material-ui/core';
import { useCustomSnackbar } from '../components/common/snackbar/Snackbar';

const createSwaggerClient = async () => {
  const swaggerOptions = {
    url: `${window.SERVER_ADDRESS}${window.API_BASE_ENDPOINT}`,
    requestInterceptor: (req) => {
      // if (user) req.headers["Authorization"] = `Bearer ${user.token}`;
      req.headers['Content-Type'] = 'application/json';
      req.headers['accept'] = 'application/json';
      window.req = req;
      console.log(req);
      return req;
    },
    responseInterceptor: (res) => {
      // console.log(res);
      return res;
    }
  };

  const swagger = await SwaggerClient(swaggerOptions)
  // update the server url to be our intended server
  swagger.spec.servers[0].url = `${window.SERVER_ADDRESS}`;
  window.swagger = swagger;
  return swagger;
}

export function useAPI() {
  const create = useCallback(createSwaggerClient, [])
  return useAsync({ fcn: create });
}

export const apiContext = createContext(null);
export const userContext = createContext({ user: null, userRoles:[], getUserRoles: ()=>{}, login: ()=>{}, logout: ()=>{} });

export function UserProvider({ children }) {
  function onLoginError(message) {
    setLoginError(message);
  }  

  const [user, getUser, resetUser, userLoading] = useUser();
  const [loginError, setLoginError] = useState();
  const [loginResponse, loginFcn] = useLogin(onLoginError);
  const logoutFcn = useLogout();

  const resetLoginError = useCallback(() => setLoginError(null), [setLoginError]);

  console.log("rendering UserProvider")
  useEffect(getUser, [Boolean(loginResponse), getUser]);

  const logout = useCallback(async () => {
    console.log("Logging out!")
    resetUser();
    logoutFcn();
  }, [resetUser, logoutFcn])

  const login = useCallback(async (username, password) => {
    console.log("Logging in!");
    await loginFcn(username, password);
    getUser();
  }, [loginFcn, getUser])

  const createValue = useCallback(() => {
    return { user: user?.userInfo, userRoles: user?.userRoles, getUser, login, loginError, resetLoginError, logout };
  }, [user, getUser, login, loginError, resetLoginError, logout ]);

  const [value, setValue] = useState(createValue());

  useEffect(() => {
    if (!userLoading) {
      setValue(createValue())
    }
  }, [userLoading, setValue, createValue])

  return (
    <userContext.Provider value={value}>
      {children}
    </userContext.Provider>
  );
}

export function APIProvider({ children }) {
  console.log("about to call useAPI")
  const [api] = useAPI();

  console.log(api, api?.requestInterceptor)

  return api ?
    (
      <apiContext.Provider value={api}>
        {children}
      </apiContext.Provider>
    ) :
    <CircularProgress />;
}

export function useAPIErrorHandler(onError) {
  const {logout} = useContext(userContext);
  const showError = useCustomSnackbar();

  function errorHandler(e) {
    const { response, message } = e;
    const { obj, status } = (response ?? {obj: {}, status: ""});
    if (status === 401) {
      logout();
    }
    else if (onError){
      onError(obj?.description ?? message);
    }
    else {
      showError(obj?.description ?? message);
    }
  }

  return useCallback(errorHandler, [logout]);
}

export function useAsync({ fcn, call = true, init = null, onError = null }) {
  const [response, setResponse] = useState(init);
  const [loading, setLoading] = useState(false);
  const errorHandler = useAPIErrorHandler(onError);

  const reset = useCallback(() => setResponse(init), [setResponse, init]);

  const f = async (...args) => {
    try {
      setLoading(true);
      const r = await fcn(...args);
      console.log(r);
      setResponse(r);
    } catch (e) {
      console.warn(e);
      errorHandler(e);
    }
    setLoading(false);
  };

  const wrapper = useCallback(f, [fcn, setResponse, setLoading, errorHandler]);
  // const [loading, lwrapper] = useLoading(wrapper, response);

  useEffect(() => {
    console.log("i'm callin");
    if (call) wrapper();
  }, [wrapper, call]);

  return [response, wrapper, reset, loading];
}

function useCallAndUnpack(fcn, unpacker = x => x) {
  return useCallback(async (...args) => {
    console.log(...args);
    const response = await fcn(...args);
    const unpacked = unpacker(response.obj, response.status);
    return unpacked;
  }, []);
}

export function unpackIOC(ioc) {
  console.log(ioc);
  ioc = { ...ioc };
  const { id, description, owner, createdBy, active,
     activeDeployment, namingName, gitProjectId,
      sourceUrl, sourceVersion, sourceVersionShort, operationInProgress, alerts } = ioc;

  let unpackedIOC = {
    id: id,
    description: description,
    owner: owner,
    createdBy: createdBy,
    activeDeployment: activeDeployment,
    active: active,
    gitProjectId: gitProjectId,
    sourceUrl: sourceUrl,
    sourceVersion: sourceVersion,
    sourceVersionShort: sourceVersionShort,
    namingName: namingName,
    operationInProgress: operationInProgress,
    alerts: alerts
  };

  return unpackedIOC;
}

export function unpackIocInfo(ioc) {
  console.log(ioc);
  ioc = { ...ioc };
  const { id, owner, active, activeDeployment, namingName, sourceUrl, sourceVersion, sourceVersionShort, alert} = ioc;

  let unpackedIOC = {
    id: id,
    owner: owner,
    activeDeployment: activeDeployment,
    active: active,
    sourceUrl: sourceUrl,
    sourceVersion: sourceVersion,
    sourceVersionShort: sourceVersionShort,
    namingName: namingName,
    alert: alert
  };

  return unpackedIOC;
}

const emptyIocListResponse = {
  totalCount: 0,
    pageNumber: 0,
    limit: 0,
    iocList : []
}

export function unpackIOCList(iocs) {
  
  let iocArray = iocs.iocList.map((ioc) => unpackIocInfo(ioc));

  let unpackedIOCList = {
    totalCount: iocs.totalCount,
    pageNumber: iocs.pageNumber,
    limit: iocs.limit,
    iocList : iocArray
  };

  return unpackedIOCList;
}

export function useIOCList() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.listIocs, unpackIOCList);
  return useAsync({ fcn: method, init: emptyIocListResponse });
}

export function useIOCSearch() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (owner, query, page, limit) => api.apis.IOCs.listIocs({ owner: owner, query: query, page:page, limit: limit }),
    unpackIOCList
  );
  return useAsync({ fcn: method, call: false, init: emptyIocListResponse });
}

export function useIOC(id) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.getIoc, unpackIOC);
  const boundMethod = useCallback(method.bind(null, { iocId: id }), [id])
  return useAsync({ fcn:  boundMethod});
}

export function useCreateIOC(onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    body => api.apis.IOCs.createIoc({}, { requestBody: body }),
    unpackIOC
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackDeployment(deployment) {
  const d = { ...deployment };
  console.log(d);
  return d;
}

export function unpackDeploymentList(deployments) {
  
  let deploymentArr = deployments.deployments.map(d => unpackDeployment(d));
  
  let unpackedDeploymentList = {
    totalCount: deployments.totalCount,
    pageNumber: deployments.pageNumber,
    limit: deployments.limit,
    deploymentList : deploymentArr
  };

  return unpackedDeploymentList;
}

const emptyDeploymentListResponse = {
  totalCount: 0,
    pageNumber: 0,
    limit: 0,
    deploymentList : []
}

export function useDeploymentListForIOC() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (iocId, page, limit) => api.apis.Deployments.listDeployments({ iocId: iocId, page: page, limit: limit }),
    unpackDeploymentList
  );
  return useAsync({ fcn: method, call: false, init: emptyDeploymentListResponse });
}

export function unpackCommand(command) {
  const c = { ...command };
  console.log(c);
  return c;
}

const emptyCommandListResponse = {
  totalCount: 0,
    pageNumber: 0,
    limit: 0,
    commandList : []
}

export function unpackCommandList(commandList) {
  let commandArr = commandList.commands.map(c => unpackCommand(c));

  let unpackedCommandList = {
    totalCount: commandList.totalCount,
    pageNumber: commandList.pageNumber,
    limit: commandList.limit,
    commandList : commandArr
  };

  return unpackedCommandList;
}

export function useCommandList() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (deploymentId, page, limit) => api.apis.Deployments.listCommandsByDeployment({deploymentId : deploymentId, page: page, limit: limit}), unpackCommandList)
  return useAsync({ fcn: method, call: false, init: emptyCommandListResponse });
}

export function unpackOngoingCommand(command) {
  if (command) {
    return { ...command };
  }
  return null;
}

export function useOngoingCommand(iocId) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Deployments.ongoingCommandByIoc, unpackOngoingCommand)
  return useAsync({ fcn: method.bind(null, { iocId: iocId }), call: false });
}

export function useDeploymentSearch() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (query) => api.apis.Deployments.listDeployments({ query: query }),
    unpackDeploymentList
  );
  return useAsync({ fcn: method, call: false, init: emptyDeploymentListResponse });
}

export function useDeployment(id) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Deployments.fetchDeployment, unpackDeployment)
  const boundMethod = useCallback(method.bind(null, { deploymentId: id }), [id])
  return useAsync({ fcn: boundMethod });
}

export function useDeploymentById() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (deploymentId) => api.apis.Deployments.fetchDeployment({ deploymentId: deploymentId }),
    unpackDeployment
  );
  return useAsync({ fcn: method, call: false, init: null });
}

export function useCommandById() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (commandId) => api.apis.Deployments.fetchCommand({ commandId: commandId }),
    unpackCommand
  );
  return useAsync({ fcn: method, call: false, init: null });
}

export function unpackDeploymentJob(job) {
  return { ...job };
}

export function useDeploymentJob(awxJobId, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Deployments.fetchDeploymentJobDetails, unpackDeploymentJob)
  const boundMethod = useCallback(method.bind(null, { awxJobId: awxJobId }), [awxJobId])
  return useAsync({ fcn: boundMethod, call: true, onError: onError });
}

export function unpackCommandJob(job) {
  return { ...job };
}

export function useCommandJob(awxCommandId) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Deployments.fetchCommandDetails, unpackCommandJob)
  const boundMethod = useCallback(method.bind(null, { awxCommandId: awxCommandId }), [awxCommandId])
  return useAsync({ fcn: boundMethod, call: true });
}

export function useUpdateAndDeployIoc(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    body => api.apis.Deployments.updateAndDeployIoc({ iocId: id }, { requestBody: body }),
    unpackDeployment
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function useCreateUndeployment(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    body => api.apis.Deployments.createUndeployment({iocId: id}, { requestBody: body }),
    unpackDeployment
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackHost(host) {
  return { ...host };
}

export function unpackHostList(hosts) {
  return hosts.csEntryHosts.map(h => unpackHost(h));
}

export function useHostList() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Hosts.listHosts, unpackHostList)
  return useAsync({ fcn: method, init: [] });
}

export function useHost(id) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Hosts.findHostById, unpackHost)
  const boundMethod = useCallback(method.bind(null, { hostCSEntryId: id }), [id]);
  return useAsync({ fcn: boundMethod});
}

export function useAvailableHost(id) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Hosts.hostDetail, unpackHost)
  const boundMethod = useCallback(method.bind(null, { hostId: id }), method, id)
  return useAsync({ fcn: boundMethod});
}

export function unpackCSEntryHost(host) {
  return { ...host };
}

const emptyHostListResponse = {
  totalCount: 0,
    pageNumber: 0,
    limit: 0,
    hostList : []
}

export function unpackCSEntryHostList(hosts) {
  let hostArr = hosts.csEntryHosts.map(h => unpackCSEntryHost(h));
  
  let unpackedHostList = {
    totalCount: hosts.totalCount,
    pageNumber: hosts.pageNumber,
    limit: hosts.limit,
    hostList : hostArr
  };

  return unpackedHostList;
}

export function useCSEntrySearch() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (query, page, limit) => api.apis.Hosts.listHosts({ query: query, page: page, limit: limit }),
    unpackCSEntryHostList
  );
  return useAsync({ fcn: method, call: false, init: emptyHostListResponse });
}

const emptyHostIocListResponse = {
  totalCount: 0,
    pageNumber: 0,
    limit: 0,
    deployedList : []
}

export function unpackHostIOCList(iocs) {
  
  const deployedArr = iocs.deployedIocs.map(ioc => unpackIocInfo(ioc))
  
  let unpackedHostIocList = {
    totalCount: iocs.totalCount,
    pageNumber: iocs.pageNumber,
    limit: iocs.limit,
    deployedList : deployedArr
  };

  return unpackedHostIocList;
}

export function useHostIOCList() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (params) => api.apis.Hosts.findAssociatedIocsByHostId({ hostCSEntryId: params.hostCSEntryId, page: params.page, limit: params.limit }), 
    unpackHostIOCList);
  return useAsync({ fcn: method, call: false, init: emptyHostIocListResponse });
}

export function unpackLogin(loginResponse) {
  return { ...loginResponse };
}

export function useLogin(onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (username, password) => api.apis.Authentication.login({},{
      requestBody: {
        userName: username,
        password: password
      }
    }),
    unpackLogin
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackUser(user) {
  return { ...user };
}

export function useUser() {
  const [userInfo, getUserInfo, resetUserInfo, userInfoLoading] = useUserInfo();
  const [userRoles, getUserRoles, resetUserRoles, userRolesLoading] = useUserRoles();
  const [user, setUser] = useState();

  useEffect(() => {
    // userInfo and user Roles should:
    // - both be defined when logged in
    // - both be undefined when logged out
    const loginStateConsistent = Boolean(userInfo) === Boolean(userRoles);
    if (loginStateConsistent) {
      setUser({userInfo, userRoles});
    }
  }, [userInfo, userRoles])

  const getUser = useCallback(() => {
    getUserInfo();
    getUserRoles();
  }, [getUserInfo, getUserRoles]);

  const resetUser = useCallback(() => {
    resetUserInfo();
    resetUserRoles();
  }, [resetUserInfo, resetUserRoles]);

  const loading = userInfoLoading || userRolesLoading;

  return [user, getUser, resetUser, loading]
}


export function useUserInfo() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Git.userInfo, unpackUser);
  return useAsync({ fcn: method, call: false });
}

export function unpackUserRoles(roles) {
  return roles;
}

export function useUserRoles() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Authentication.getUserRoles, unpackUserRoles);
  return useAsync({ fcn: method, call: false });
}

export function useLogout() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Authentication.logout);
  const [, logout] = useAsync({ fcn: method, call: false });
  return logout;
}

export function useStartIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Deployments.startIoc);
  const boundMethod = useCallback(method.bind(null, {iocId: id}), [id])
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function useStopIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Deployments.stopIoc);
  const boundMethod = useCallback(method.bind(null, {iocId: id}), [id])
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function useRenewToken() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Authentication.tokenRenew);

  const [, renewToken] = useAsync({ fcn: method, call: false });
  return renewToken;
}

export function unpackLokilog(logData) {
  return { ...logData };
}

export function useLokiSysLog() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack((hostName, timeRange) => api.apis.Loki.fetchSyslogLines({ hostName: hostName, timeRange: timeRange }), unpackLokilog)
  return useAsync({ fcn: method, call: false });
}


export function useLokiProcServLog() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack((hostName, iocName, timeRange) => api.apis.Loki.fetchProcServLogLines({ hostName: hostName, iocName:iocName, timeRange: timeRange }), unpackLokilog)

  return useAsync({ fcn: method, call: false });
}

export function unpackStatistics(statistics) {
  return { ...statistics };
}

export function useStatistics() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Statistics.generalStatistics, unpackStatistics)

  return useAsync({ fcn: method, call: false });
}

export function unpackNaming(naming) {
  return { ...naming };
}

export function unpackNamingList(naming) {
  return naming.map(h => unpackNaming(h));
}

export function useNamingNames() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack((iocName) => api.apis.Naming.fetchIOCByName({iocName: iocName}), unpackNamingList)

  return useAsync({ fcn: method, call: false, init: [] });
}

export function unpackUserList(users) {
  return users.toString().split(",");
}

export function useUserNames() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack((query) => api.apis.Authentication.getToolUsers({query: query}), unpackUserList)

  return useAsync({ fcn: method, call: false, init: [] });
}

export function unpackDeploymentStatistics(statistics) {
  return { ...statistics };
}

export function useDeploymentStatistics() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Statistics.deploymentStatistics, unpackDeploymentStatistics)

  return useAsync({ fcn: method, call: false });
}

export function unpackGitReference(reference) {
  return { ...reference };
}

export function unpackTagAndCommitIdList(tagList) {
  return tagList.map(t => unpackGitReference(t));
}

export function useTagsAndCommitIds() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack((gitProjectId) => api.apis.Git.listTagsAndCommitIds({projectId : gitProjectId}), unpackTagAndCommitIdList)
  return useAsync({ fcn: method, call: false, init: [] });
}

export function unpackIOCStatistics(statistics) {
  return { ...statistics };
}

export function useIOCStatistics() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Statistics.iocStatistics, unpackIOCStatistics)

  return useAsync({ fcn: method, call: false });
}

export function unpackCurrentlyActiveIOCS(statistics) {
  return { ...statistics };
}

export function useCurrentlyActiveIOCs() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Statistics.currentlyActiveIocs, unpackCurrentlyActiveIOCS)

  return useAsync({ fcn: method, call: false });
}

export function unpackActiveIOCHistory(statistics) {
  return { ...statistics };
}

export function useActiveIOCHistory() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Statistics.activeIocHistory, unpackActiveIOCHistory)

  return useAsync({ fcn: method, call: false });
}

export function unpackDeployedIOCStat(statistics) {
  return { ...statistics };
}

export function useDeployedIOCStatistics() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Statistics.iocDeploymentHistory, unpackDeployedIOCStat)

  return useAsync({ fcn: method, call: false });
}

export function unpackGitProject(project) {
  return { ...project };
}

export function unpackAllowedGitProjects(projectList) {
  return projectList.map(p => unpackGitProject(p));
}

export function useAllowedGitProjects() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack((repoUrl) => api.apis.Git.listProjects(), unpackAllowedGitProjects)
  return useAsync({ fcn: method, call: false, init: [] });
}

export function useDeleteIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.deleteIoc, unpackDeleteIocResponse);
  const boundMethod = useCallback(method.bind(null, {iocId: id}), [id])
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function unpackDeleteIocResponse(_, status) {
  if (status === 204) {
    return "Successful delete";
  }
}

export function useUpdateIoc(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    body => api.apis.IOCs.updateIoc({ iocId: id }, { requestBody: body }),
    unpackUpdateIoc
  );
  return useAsync({ fcn: method, call: false, onError: onError  });
}

export function unpackUpdateIoc(updateIoc) {
  const d = { ...updateIoc };
  console.log(d);
  return d;
}