import { Button, LinearProgress, Grid, Typography, Tooltip } from "@material-ui/core";
import { makeStyles } from '@material-ui/core/styles';
import React, { useState, useEffect, useRef, useContext } from "react";
import { useStartIOC, useStopIOC } from "../../api/SwaggerApi";
import { CommandJobStatus } from "./CommandJobStatus";
import { SimpleModal } from "../../components/common/SimpleModal/SimpleModal";
import { ConfirmationDialog } from "../dialog/ConfirmationDialog";
import { notificationContext } from "../../components/common/notification/Notifications";
import { Alert } from "@material-ui/lab";
import { theme } from "../../style/Theme";
import { initRequestParams } from "../common/Helper";
import AccessControl from "../auth/AccessControl";

const useStyles = makeStyles({
  startButton: {
      backgroundColor: '#4caf50',
      color: '#FFFFFF',
      "&:disabled": {
        backgroundColor: '#e0e0e0',
        color: '#afaba6',
      },
      "&:hover": {
        backgroundColor: '#005a00',
        color: '#FFFFFF',
      }
  },
  stopButton: {
      backgroundColor: '#aa2e25',
      color: '#FFFFFF',
      "&:disabled": {
        backgroundColor: '#e0e0e0',
        color: '#afaba6',
      },
      "&:hover": {
        backgroundColor: '#6c0000',
        color: '#FFFFFF',
      }
  },
});

export function IOCService({ ioc, currentCommand, getCommands, buttonDisabled, setButtonDisabled, setOperationQueued, commandLazyParams, commandColumnSort }) {
  const classes = useStyles(theme);

  function onError(message) {
    setOperationQueued(false);
    setButtonDisabled(false);
    setError(message);
  }  
  const [startJob, startIOC, resetStartJob] = useStartIOC(ioc.id, onError);
  const [stopJob, stopIOC, resetStopJob] = useStopIOC(ioc.id, onError);
  const [inProgress, setInProgress] = useState(false);
  const [action, setAction] = useState(false);
  const [adHocDialogOpen, setAdHocDialogOpen] = useState(false);
  const [adHocDialogTitle, setAdHocDiatlogTitle] = useState();
  const [adHocDialogDescription, setAdHocDialogDescription] = useState();
  const [command, setCommand] = useState(null);
  const [error, setError] = useState();
  const callbackRef = useRef();

  const {watchCommand} = useContext(notificationContext);

  useEffect(() => {
    if (currentCommand) {
      if (currentCommand.type === "START") {
        setAction("Start");
      }
      if (currentCommand.type === "STOP") {
        setAction("Stop");
      }

      watchCommand(currentCommand.id);
      setCommand(currentCommand);
      setInProgress(true);
    }
  }, [currentCommand, watchCommand]);

  useEffect(() => {
    if (startJob && (!command || command.id !== startJob.id)) {
      watchCommand(startJob.id);
      setCommand(startJob);
      setInProgress(false);
    } else if (stopJob && (!command || command.id !== stopJob.id)) {
      watchCommand(stopJob.id);
      setCommand(stopJob);
      setInProgress(false);
    }
  }, [startJob, stopJob, command, watchCommand, getCommands]);

  function resetUI() {
    setCommand(null);
    setAction(null);
    resetStartJob();
    resetStopJob();
    setError(null);
  }

  const start = async () => {
    resetUI();
    setAction("Start");
    setInProgress(true);
    setOperationQueued(true);
    await startIOC();
    let requestParams = initRequestParams(commandLazyParams, null, commandColumnSort);

    requestParams.deploymentId = ioc.activeDeployment?.id;
    getCommands(requestParams);
  }

  const stop = async () => {
    resetUI();
    setAction("Stop");
    setInProgress(true);
    setOperationQueued(true);
    await stopIOC();

    let requestParams = initRequestParams(commandLazyParams, null, commandColumnSort);

    requestParams.deploymentId = ioc.activeDeployment?.id;
    getCommands(requestParams);
  }

  const openStartModal = () => {
    setAdHocDiatlogTitle("Starting IOC");
    setAdHocDialogDescription(
    <>
      <Typography style={{ display: 'inline-block' }}>Are you sure want to start &nbsp;</Typography>
      <Typography style={{ fontFamily: "monospace", display: 'inline-block' }}> {ioc.name}? </Typography>
    </>);
    callbackRef.current = start;
    setAdHocDialogOpen(true);
  }

  const openStopModal = () => {
    setAdHocDiatlogTitle("Stopping IOC");
    setAdHocDialogDescription(
    <>
      <Typography style={{ display: 'inline-block' }}>Are you sure want to stop &nbsp;</Typography>
      <Typography style={{ fontFamily: "monospace", display: 'inline-block' }}> {ioc.name}? </Typography>
    </>);
    callbackRef.current = stop;
    setAdHocDialogOpen(true);
  }

  let disabledStartButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledStartButtonTitle = "There is an ongoing operation, you can not Start the IOC right now";
    }

    let disabledStopButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledStopButtonTitle = "There is an ongoing operation, you can not Stop the IOC right now";
    }

  return (
    <>
      <AccessControl
        allowedRoles={["DeploymentToolAdmin"]}
        allowedUsersWithRoles={[{ user: ioc.owner, role: "DeploymentToolIntegrator" }]}
        renderNoAccess={() => <></>}>
        <SimpleModal open={adHocDialogOpen} setOpen={setAdHocDialogOpen}>
          <ConfirmationDialog open={adHocDialogOpen} setOpen={setAdHocDialogOpen} title={adHocDialogTitle} description={adHocDialogDescription} callback={callbackRef.current} />
        </SimpleModal>

        <Grid container spacing={1}>
          {ioc.active === true ?
            <Grid item xs={1} md={1}>
              <Tooltip title={disabledStopButtonTitle}>
                <span>
                  <Button className={classes.stopButton} onClick={openStopModal} disabled={buttonDisabled || ioc.operationInProgress}>Stop</Button>
                </span>
              </Tooltip>
            </Grid>
            :
            <></>
          }
          {ioc.active === false ?
            <Grid item xs={1} md={1}>
              <Tooltip title={disabledStartButtonTitle}>
                <span>
                  <Button className={classes.startButton} onClick={openStartModal} disabled={buttonDisabled || ioc.operationInProgress}>Start</Button>
                </span>
              </Tooltip>
            </Grid>
            :
            <></>
          }
          <Grid item xs={12} md={12}>
            {error ?
              <Alert severity="error">{error}</Alert>
              :
              <>
                {command ?
                  <CommandJobStatus awxCommandId={command.awxCommandId} actionType={action} /> : inProgress ? <LinearProgress color="primary" /> : <></>
                }
              </>
            }
          </Grid>
        </Grid>
      </AccessControl>
    </>
  )
}
