import React, { useEffect, useState } from 'react';
import { Grid, Typography, Card, CardContent, Container, Link as MuiLink} from '@material-ui/core';
import { makeStyles, createMuiTheme } from '@material-ui/core/styles';
import { DeploymentStepper } from "./DeploymentStepper"
import { KeyValueTable } from "../common/KeyValueTable/KeyValueTable";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { DeploymentBadge } from './DeploymentBadge';
import { DeploymentJobOutput } from './DeploymentJobOutput';
import { Alert } from '@material-ui/lab';
import { Link } from 'react-router-dom';
import { formatDate } from '../common/Helper';
import GitRefLink from '../IOC/GitRefLink';
import AccessControl from '../auth/AccessControl';

const useStyles = makeStyles({
  root: {
    width: '100%'
  },
  actions: {
    justifyContent: 'flex-end',
  },
});

export function DeploymentDetails({ deployment, getDeployment }) {
  const [alert, setAlert] = useState((<Alert severity="info" variant="standard">The {deployment.undeployment ? "undeployment" : "deployment"} is still in progress.</Alert>));
  const [updateOutput, setUpdateOutput] = useState(true);

  useEffect(() => {
    if (deployment) {
      if (!(["successful", "failed"].includes(deployment.status.toLowerCase()))) {
        if (["queued", "pending"].includes(deployment.status.toLowerCase())) {
          setAlert(<Alert severity="info" variant="standard">The {deployment.undeployment ? "undeployment" : "deployment"} is queued.</Alert>);
        } else if (["running"].includes(deployment.status.toLowerCase())) {
          setAlert(<Alert severity="info" variant="standard">The {deployment.undeployment ? "undeployment" : "deployment"} is running.</Alert>);
        }
        setTimeout(getDeployment, 5000);
        setUpdateOutput(true);
      }
      else {
        setUpdateOutput(false);
        let alert = (deployment.status.toLowerCase() === "failed") ?
          (<Alert severity="error" variant="standard">The {deployment.undeployment ? "undeployment" : "deployment"} had failures.</Alert>)
          : (<Alert severity="success" variant="standard"><Container>The {deployment.undeployment ? "undeployment" : "deployment"} was successful.</Container></Alert>);
        setAlert(alert);
      }
    }
  }, [deployment, getDeployment]);

  const deploymentDetails = {
    "Git reference": <GitRefLink ioc={deployment} />,
    host: 
    <Typography>
      {deployment.host.csEntryId ?
        <Link to={`/hosts/${deployment.host.csEntryId}`}>{deployment.host.fqdn}</Link>
        :
        <>{deployment.host.fqdn}</>
      }
    </Typography>,
    comment: deployment.comment,
    repository: <Typography><MuiLink href={deployment.sourceUrl} target="_blank" rel="noreferrer">{deployment.sourceUrl}</MuiLink></Typography>,
    "AWX job link": <Typography><MuiLink href={deployment.awxJobUrl} target="_blank" rel="noreferrer">{deployment.awxJobUrl}</MuiLink></Typography>,
    "start time": deployment.startDate ? formatDate(deployment.startDate) : "-",
    "end time": deployment.endDate ? formatDate(deployment.endDate) : "-"
  }

  const theme = createMuiTheme();
  const classes = useStyles(theme);

  return (
    <Grid container spacing={1}>
      <Grid item xs={12} md={12}>
        <SimpleAccordion defaultExpanded summary={<DeploymentBadge deployment={deployment} />}>
          <KeyValueTable obj={deploymentDetails} variant="overline" />
        </SimpleAccordion>
      </Grid>
      <Grid item xs={12} md={12}>
        <Card className={classes.root}>
          <CardContent>
            <div className={classes.stepperContainer}>
              <DeploymentStepper activeStep={["created", "pending", "running", "successful"].indexOf(deployment.status.toLowerCase())} deploymentStatus={deployment.status}
                deploymentStart={deployment.startDate} undeployment={deployment.undeployment} />
            </div>
          </CardContent>
        </Card>
      </Grid>
      {alert && <Grid item xs={12}>
          {alert}
        </Grid>}
      <AccessControl allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}>
        <Grid item xs={12} md={12} style={{ paddingBottom: 0 }}>
          {deployment.awxJobId ?
            <SimpleAccordion summary="Ansible Job Output" defaultExpanded>
              <Container>
                <DeploymentJobOutput awxJobId={deployment.awxJobId} update={updateOutput} />
              </Container>
            </SimpleAccordion>
            :
            <></>
          }
        </Grid>
      </AccessControl>
    </Grid>
  );
}
